@@ -5,6 +5,17 @@
 #' @name diagnostics
 NULL
 
+
+#' Check if lintr is 2.0.1.9000 above. A number of features relies on it.
+#' @noRd
+lintr_is_new_enough <- function() {
+    return(
+        utils::packageVersion("lintr") >= "2.0.1.9000" &&
+            "text" %in% names(formals(lintr::lint))
+    )
+}
+
+
 DiagnosticSeverity <- list(
     Error = 1,
     Warning = 2,
@@ -13,7 +24,7 @@ DiagnosticSeverity <- list(
 )
 
 #' @rdname diagnostics
-#' @keywords internal
+#' @noRd
 diagnostic_range <- function(result, content) {
     line <- result$line_number - 1
     column <- result$column_number - 1
@@ -34,7 +45,7 @@ diagnostic_range <- function(result, content) {
 }
 
 #' @rdname diagnostics
-#' @keywords internal
+#' @noRd
 diagnostic_severity <- function(result) {
     switch(result$type,
         error = DiagnosticSeverity$Error,
@@ -44,7 +55,7 @@ diagnostic_severity <- function(result) {
 }
 
 #' @rdname diagnostics
-#' @keywords internal
+#' @noRd
 diagnostic_from_lint <- function(result, content) {
     list(
         range = diagnostic_range(result, content),
@@ -55,7 +66,7 @@ diagnostic_from_lint <- function(result, content) {
 }
 
 #' Find the lintr config file
-#' @keywords internal
+#' @noRd
 find_config <- function(filename) {
     # instead of calling `lintr:::find_config` directly
     # since CRAN doesn't like :::.
@@ -65,8 +76,8 @@ find_config <- function(filename) {
 #' Run diagnostic on a file
 #'
 #' Lint and diagnose problems in a file.
-#' @keywords internal
-diagnose_file <- function(uri, content) {
+#' @noRd
+diagnose_file <- function(uri, content, cache = FALSE) {
     if (length(content) == 0) {
         return(list())
     }
@@ -79,36 +90,42 @@ diagnose_file <- function(uri, content) {
     }
 
     path <- path_from_uri(uri)
-    linter_file <- find_config(path)
-    if (is.null(linter_file)) {
-        linters <- getOption("languageserver.default_linters", NULL)
-    } else {
-        linters <- NULL
-        op <- options(lintr.linter_file = linter_file)
-        on.exit(options(op))
-    }
 
     if (length(content) == 1) {
         content <- c(content, "")
     }
 
-    text <- paste0(content, collapse = "\n")
-    diagnostics <- lapply(
-        lintr::lint(text, linters = linters), diagnostic_from_lint, content = content)
+    if (lintr_is_new_enough()) {
+        lints <- lintr::lint(path, cache = cache, text = content)
+    } else {
+        # TODO: remove it once new version of lintr is released
+        linter_file <- find_config(path)
+        if (!is.null(linter_file)) {
+            op <- options(lintr.linter_file = linter_file)
+            on.exit(options(op))
+        }
+        text <- paste0(content, collapse = "\n")
+        lints <- lintr::lint(text, cache = cache)
+    }
+
+    diagnostics <- lapply(lints, diagnostic_from_lint, content = content)
     names(diagnostics) <- NULL
     diagnostics
 }
 
-
 diagnostics_callback <- function(self, uri, version, diagnostics) {
     if (is.null(diagnostics) || !self$workspace$documents$has(uri)) return(NULL)
-    logger$info("diagnostics_callback called:", list(uri = uri, version = version))
-    # logger$info("diagnostics:", diagnostics)
+    logger$info("diagnostics_callback called:", list(
+        uri = uri,
+        version = version,
+        diagnostics = diagnostics
+    ))
     self$deliver(
         Notification$new(
             method = "textDocument/publishDiagnostics",
             params = list(
                 uri = uri,
+                version = version,
                 diagnostics = diagnostics
             )
         )
@@ -121,7 +138,7 @@ diagnostics_task <- function(self, uri, document, delay = 0) {
     content <- document$content
     create_task(
         target = package_call(diagnose_file),
-        args = list(uri = uri, content = content),
+        args = list(uri = uri, content = content, cache = lsp_settings$get("lint_cache")),
         callback = function(result) diagnostics_callback(self, uri, version, result),
         error = function(e) {
             logger$info("diagnostics_task:", e)
